@@ -30,16 +30,14 @@ responsible for resolving identifier crossreferences
 Markup errors are represented using L{ParseError}s.  These exception
 classes record information about the cause, location, and severity of
 each error.
-
-@sort: ParsedDocstring, Field, DocstringLinker
-@group Errors and Warnings: ParseError
 """
 __docformat__ = 'epytext en'
 
+from typing import List, Optional, Sequence, Union
 import re
 
-from six import text_type
-from twisted.web.template import XMLString, flattenString
+from twisted.python.failure import Failure
+from twisted.web.template import Tag, XMLString, flattenString
 
 ##################################################
 ## Contents
@@ -58,20 +56,29 @@ class ParsedDocstring:
     """
     A standard intermediate representation for parsed docstrings that
     can be used to generate output.  Parsed docstrings are produced by
-    markup parsers such as L{pydoctor.epydoc.epytext.parse_docstring()}
-    or L{pydoctor.epydoc.restructuredtext.parse_docstring()}.
+    markup parsers such as L{pydoctor.epydoc.markup.epytext.parse_docstring()}
+    or L{pydoctor.epydoc.markup.restructuredtext.parse_docstring()}.
 
-    Subclasses must implement L{to_stan()}.
+    Subclasses must implement L{has_body()} and L{to_stan()}.
     """
 
-    def __init__(self, fields):
+    def __init__(self, fields: Sequence['Field']):
         self.fields = fields
         """
         A list of L{Field}s, each of which encodes a single field.
         The field's bodies are encoded as C{ParsedDocstring}s.
         """
 
-    def to_stan(self, docstring_linker):
+    @property
+    def has_body(self) -> bool:
+        """Does this docstring have a non-empty body?
+
+        The body is the part of the docstring that remains after the fields
+        have been split off.
+        """
+        raise NotImplementedError()
+
+    def to_stan(self, docstring_linker: 'DocstringLinker') -> Tag:
         """
         Translate this docstring to a Stan tree.
 
@@ -81,7 +88,6 @@ class ParsedDocstring:
 
         @param docstring_linker: An HTML translator for crossreference
             links into and out of the docstring.
-        @type docstring_linker: L{DocstringLinker}
         @return: The docstring presented as a tree.
         """
         raise NotImplementedError()
@@ -92,33 +98,31 @@ _RE_CONTROL = re.compile((
     ) + ']'
     ).encode())
 
-def html2stan(html):
+def html2stan(html: Union[bytes, str]) -> Tag:
     """
     Convert an HTML string to a Stan tree.
 
     @param html: An HTML fragment; multiple roots are allowed.
-    @type html: C{string}
     @return: The fragment as a tree with a transparent root node.
     """
-    if isinstance(html, text_type):
+    if isinstance(html, str):
         html = html.encode('utf8')
 
     html = _RE_CONTROL.sub(lambda m:b'\\x%02x' % ord(m.group()), html)
-    stan = XMLString(b'<div>%s</div>' % html).load()[0]
+    stan: Tag = XMLString(b'<div>%s</div>' % html).load()[0]
     assert stan.tagName == 'div'
     stan.tagName = ''
     return stan
 
-def flatten(stan):
+def flatten(stan: Tag) -> str:
     """
     Convert a document fragment from a Stan tree to HTML.
 
     @param stan: Document fragment to flatten.
     @return: An HTML string representation of the C{stan} tree.
-    @rtype: C{str}
     """
-    ret = []
-    err = []
+    ret: List[bytes] = []
+    err: List[Failure] = []
     flattenString(None, stan).addCallback(ret.append).addErrback(err.append)
     if err:
         raise err[0].value
@@ -143,40 +147,36 @@ class Field:
     Tags are automatically downcased and stripped; and arguments are
     automatically stripped.
     """
-    def __init__(self, tag, arg, body, lineno):
+
+    def __init__(self, tag: str, arg: Optional[str], body: ParsedDocstring, lineno: int):
         self._tag = tag.lower().strip()
-        if arg is None: self._arg = None
-        else: self._arg = arg.strip()
+        self._arg = None if arg is None else arg.strip()
         self._body = body
         self.lineno = lineno
 
-    def tag(self):
+    def tag(self) -> str:
         """
         @return: This field's tag.
-        @rtype: C{string}
         """
         return self._tag
 
-    def arg(self):
+    def arg(self) -> Optional[str]:
         """
-        @return: This field's argument, or C{None} if this field has
-            no argument.
-        @rtype: C{string} or C{None}
+        @return: This field's argument, or C{None} if this field has no argument.
         """
         return self._arg
 
-    def body(self):
+    def body(self) -> ParsedDocstring:
         """
         @return: This field's body.
-        @rtype: L{ParsedDocstring}
         """
         return self._body
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         if self._arg is None:
-            return '<Field @%s: ...>' % self._tag
+            return f'<Field @{self._tag}: ...>'
         else:
-            return '<Field @%s %s: ...>' % (self._tag, self._arg)
+            return f'<Field @{self._tag} {self._arg}: ...>'
 
 ##################################################
 ## Docstring Linker (resolves crossreferences)
@@ -188,15 +188,28 @@ class DocstringLinker:
     target URL for crossreference links.
     """
 
-    def resolve_identifier_xref(self, identifier):
+    def resolve_identifier(self, identifier: str) -> Optional[str]:
         """
-        Resolve a crossreference link to a Python identifier.
+        Resolve a Python identifier.
+        This will resolve the identifier like Python itself would.
 
-        @type identifier: C{string}
         @param identifier: The name of the Python identifier that
             should be linked to.
-        @rtype: C{string}
-        @return: The URL of the target
+        @return: The URL of the target, or L{None} if not found.
+        """
+        raise NotImplementedError()
+
+    def resolve_identifier_xref(self, identifier: str, lineno: int) -> str:
+        """
+        Resolve a crossreference link to a Python identifier.
+        This will resolve the identifier to any reasonable target,
+        even if it has to look in places where Python itself would not.
+
+        @param identifier: The name of the Python identifier that
+            should be linked to.
+        @param lineno: The line number within the docstring at which the
+            crossreference is located.
+        @return: The URL of the target.
         @raise LookupError: If C{identifier} could not be resolved.
         """
         raise NotImplementedError()
@@ -209,75 +222,68 @@ class DocstringLinker:
 class ParseError(Exception):
     """
     The base class for errors generated while parsing docstrings.
-
-    @ivar _linenum: The line on which the error occured within the
-        docstring.  The linenum of the first line is 0.
-    @type _linenum: C{int}
-    @ivar _descr: A description of the error.
-    @type _descr: C{string}
-    @ivar _fatal: True if this is a fatal error.
-    @type _fatal: C{bool}
     """
-    def __init__(self, descr, linenum=None, is_fatal=True):
+
+    def __init__(self,
+            descr: str,
+            linenum: Optional[int] = None,
+            is_fatal: bool = True
+            ):
         """
-        @type descr: C{string}
         @param descr: A description of the error.
-        @type linenum: C{int}
         @param linenum: The line on which the error occured within
             the docstring.  The linenum of the first line is 0.
-        @type is_fatal: C{bool}
         @param is_fatal: True if this is a fatal error.
         """
         self._descr = descr
         self._linenum = linenum
         self._fatal = is_fatal
 
-    def is_fatal(self):
+    def is_fatal(self) -> bool:
         """
         @return: true if this is a fatal error.  If an error is fatal,
             then epydoc should ignore the output of the parser, and
             parse the docstring as plaintext.
-        @rtype: C{bool}
         """
         return self._fatal
 
-    def linenum(self):
+    def linenum(self) -> Optional[int]:
         """
         @return: The line number on which the error occured (including
         any offset).  If the line number is unknown, then return
         C{None}.
-        @rtype: C{int} or C{None}
         """
         if self._linenum is None: return None
         else: return self._linenum + 1
 
-    def descr(self):
+    def descr(self) -> str:
+        """
+        @return: A description of the error.
+        """
         return self._descr
 
-    def __str__(self):
+    def __str__(self) -> str:
         """
         Return a string representation of this C{ParseError}.  This
         multi-line string contains a description of the error, and
         specifies where it occured.
 
         @return: the informal representation of this C{ParseError}.
-        @rtype: C{string}
         """
         if self._linenum is not None:
-            return 'Line %d: %s' % (self._linenum + 1, self.descr())
+            return f'Line {self._linenum + 1:d}: {self.descr()}'
         else:
             return self.descr()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """
         Return the formal representation of this C{ParseError}.
         C{ParseError}s have formal representations of the form::
            <ParseError on line 12>
 
         @return: the formal representation of this C{ParseError}.
-        @rtype: C{string}
         """
         if self._linenum is None:
             return '<ParseError on unknown line>'
         else:
-            return '<ParseError on line %d>' % (self._linenum + 1)
+            return f'<ParseError on line {self._linenum + 1:d}>'
