@@ -89,15 +89,39 @@ Description::
 
    <!ELEMENT symbol (#PCDATA)>
 
-@var SYMBOLS: A list of the of escape symbols that are supported
-      by epydoc.  Currently the following symbols are supported:
-<<<SYMBOLS>>>
+@var SYMBOLS: A list of the of escape symbols that are supported by epydoc.  Currently the following symbols are supported ::
+
+    # Arrows
+    '<-', '->', '^', 'v',
+
+    # Greek letters
+    'alpha', 'beta', 'gamma', 'delta', 'epsilon', 'zeta',
+    'eta', 'theta', 'iota', 'kappa', 'lambda', 'mu',
+    'nu', 'xi', 'omicron', 'pi', 'rho', 'sigma',
+    'tau', 'upsilon', 'phi', 'chi', 'psi', 'omega',
+    'Alpha', 'Beta', 'Gamma', 'Delta', 'Epsilon', 'Zeta',
+    'Eta', 'Theta', 'Iota', 'Kappa', 'Lambda', 'Mu',
+    'Nu', 'Xi', 'Omicron', 'Pi', 'Rho', 'Sigma',
+    'Tau', 'Upsilon', 'Phi', 'Chi', 'Psi', 'Omega',
+
+    # HTML character entities
+    'larr', 'rarr', 'uarr', 'darr', 'harr', 'crarr',
+    'lArr', 'rArr', 'uArr', 'dArr', 'hArr',
+    'copy', 'times', 'forall', 'exist', 'part',
+    'empty', 'isin', 'notin', 'ni', 'prod', 'sum',
+    'prop', 'infin', 'ang', 'and', 'or', 'cap', 'cup',
+    'int', 'there4', 'sim', 'cong', 'asymp', 'ne',
+    'equiv', 'le', 'ge', 'sub', 'sup', 'nsub',
+    'sube', 'supe', 'oplus', 'otimes', 'perp',
+
+    # Alternate (long) names
+    'infinity', 'integral', 'product',
+    '>=', '<=',
+
 """
 # Note: the symbol list is appended to the docstring automatically,
 # below.
 
-from __future__ import print_function
-
 __docformat__ = 'epytext en'
 
 # Code organization..
@@ -107,11 +131,12 @@ __docformat__ = 'epytext en'
 #   4. helpers
 #   5. testing
 
+from typing import Any, Optional, Sequence, Union
 import re
-import six
+
 from twisted.web.template import CharRef, Tag, tags
 from pydoctor.epydoc.doctest import colorize_doctest
-from pydoctor.epydoc.markup import Field, ParseError, ParsedDocstring
+from pydoctor.epydoc.markup import DocstringLinker, Field, ParseError, ParsedDocstring
 
 ##################################################
 ## DOM-Like Encoding
@@ -143,17 +168,17 @@ class Element:
         """
         Return a string representation of this element, using XML
         notation.
-        @bug: Doesn't escape '<' or '&' or '>'.
+        @note: Doesn't escape '<' or '&' or '>', so the result is only XML-like
+            and cannot actually be parsed as XML.
         """
-        attribs = ''.join([' %s=%r' % t for t in self.attribs.items()])
-        return ('<%s%s>' % (self.tag, attribs) +
-                ''.join([str(child) for child in self.children]) +
-                '</%s>' % self.tag)
+        attribs = ''.join(f' {k}={v!r}' for k, v in self.attribs.items())
+        content = ''.join(str(child) for child in self.children)
+        return f'<{self.tag}{attribs}>{content}</{self.tag}>'
 
     def __repr__(self):
-        attribs = ''.join([', %s=%r' % t for t in self.attribs.items()])
-        args = ''.join([', %r' % c for c in self.children])
-        return 'Element(%s%s%s)' % (self.tag, args, attribs)
+        attribs = ''.join(f', {k}={v!r}' for k, v in self.attribs.items())
+        args = ''.join(f', {c!r}' for c in self.children)
+        return f'Element({self.tag}{args}{attribs})'
 
 ##################################################
 ## Constants
@@ -224,13 +249,13 @@ _LINK_COLORIZING_TAGS = ['link', 'uri']
 ## Structuring (Top Level)
 ##################################################
 
-def parse(str, errors = None):
+def parse(text, errors = None):
     """
     Return a DOM tree encoding the contents of an epytext string.  Any
     errors generated during parsing will be stored in C{errors}.
 
-    @param str: The epytext string to parse.
-    @type str: C{string}
+    @param text: The epytext string to parse.
+    @type text: C{str}
     @param errors: A list where any errors generated during parsing
         will be stored.  If no list is specified, then fatal errors
         will generate exceptions, and non-fatal errors will be
@@ -249,11 +274,11 @@ def parse(str, errors = None):
         raise_on_error = False
 
     # Preprocess the string.
-    str = re.sub('\015\012', '\012', str)
-    str = str.expandtabs()
+    text = re.sub('\015\012', '\012', text)
+    text = text.expandtabs()
 
     # Tokenize the input string.
-    tokens = _tokenize(str, errors)
+    tokens = _tokenize(text, errors)
 
     # Have we encountered a field yet?
     encountered_field = False
@@ -277,8 +302,8 @@ def parse(str, errors = None):
     for token in tokens:
         # Uncomment this for debugging:
         #print('%s: %s\n%s: %s\n' %
-        #       (''.join(['%-11s' % (t and t.tag) for t in stack]),
-        #        token.tag, ''.join(['%-11s' % i for i in indent_stack]),
+        #       (''.join('%-11s' % (t and t.tag) for t in stack),
+        #        token.tag, ''.join('%-11s' % i for i in indent_stack),
         #        token.indent))
 
         # Pop any completed blocks off the stack.
@@ -304,7 +329,7 @@ def parse(str, errors = None):
         elif token.tag == Token.BULLET:
             _add_list(doc, token, stack, indent_stack, errors)
         else:
-            assert 0, 'Unknown token type: '+token.tag
+            raise AssertionError(f"Unknown token type: {token.tag}")
 
         # Check if the DOM element we just added was a field..
         if stack[-1].tag == 'field':
@@ -420,7 +445,7 @@ def _add_list(doc, bullet_token, stack, indent_stack, errors):
     elif bullet_token.contents[-1] == ':':
         list_type = 'fieldlist'
     else:
-        raise AssertionError('Bad Bullet: %r' % bullet_token.contents)
+        raise AssertionError(f'Bad Bullet: {bullet_token.contents!r}')
 
     # Is this a new list?
     newlist = False
@@ -615,7 +640,7 @@ class Token:
             C{Token}s have formal representaitons of the form::
                 <Token: para at line 12>
         """
-        return '<Token: %s at line %s>' % (self.tag, self.startline)
+        return f'<Token: {self.tag} at line {self.startline}>'
 
     def to_dom(self, doc):
         """
@@ -903,13 +928,13 @@ def _tokenize_para(lines, start, para_indent, tokens, errors):
     tokens.append(Token(Token.PARA, start, contents, para_indent))
     return linenum
 
-def _tokenize(str, errors):
+def _tokenize(text, errors):
     """
     Split a given formatted docstring into an ordered list of
     C{Token}s, according to the epytext markup rules.
 
-    @param str: The epytext string
-    @type str: C{string}
+    @param text: The epytext string
+    @type text: C{str}
     @param errors: A list where any errors generated during parsing
         will be stored.  If no list is specified, then errors will
         generate exceptions.
@@ -918,7 +943,7 @@ def _tokenize(str, errors):
     @rtype: C{list} of L{Token}
     """
     tokens = []
-    lines = str.split('\n')
+    lines = text.split('\n')
 
     # Scan through the lines, determining what @type of token we're
     # dealing with, and tokenizing it, as appropriate.
@@ -986,7 +1011,7 @@ def _colorize(doc, token, errors, tagName='para'):
     @return: a DOM C{Element} encoding the given paragraph.
     @returntype: C{Element}
     """
-    str = token.contents
+    text = token.contents
 
     # Maintain a stack of DOM elements, containing the ancestors of
     # the text currently being analyzed.  New elements are pushed when
@@ -1005,7 +1030,7 @@ def _colorize(doc, token, errors, tagName='para'):
     # to the next open or close brace.
     start = 0
     while 1:
-        match = _BRACE_RE.search(str, start)
+        match = _BRACE_RE.search(text, start)
         if match is None: break
         end = match.start()
 
@@ -1016,19 +1041,19 @@ def _colorize(doc, token, errors, tagName='para'):
         # and convert them to literal braces once we find the matching
         # close-brace.
         if match.group() == '{':
-            if (end>0) and 'A' <= str[end-1] <= 'Z':
+            if (end>0) and 'A' <= text[end-1] <= 'Z':
                 if (end-1) > start:
-                    stack[-1].children.append(str[start:end-1])
-                if str[end-1] not in _COLORIZING_TAGS:
+                    stack[-1].children.append(text[start:end-1])
+                if text[end-1] not in _COLORIZING_TAGS:
                     estr = "Unknown inline markup tag."
                     errors.append(ColorizingError(estr, token, end-1))
                     stack.append(Element('unknown'))
                 else:
-                    tag = _COLORIZING_TAGS[str[end-1]]
+                    tag = _COLORIZING_TAGS[text[end-1]]
                     stack.append(Element(tag))
             else:
                 if end > start:
-                    stack[-1].children.append(str[start:end])
+                    stack[-1].children.append(text[start:end])
                 stack.append(Element('litbrace'))
             openbrace_stack.append(end)
             stack[-2].children.append(stack[-1])
@@ -1044,12 +1069,12 @@ def _colorize(doc, token, errors, tagName='para'):
 
             # Add any remaining text.
             if end > start:
-                stack[-1].children.append(str[start:end])
+                stack[-1].children.append(text[start:end])
 
             # Special handling for symbols:
             if stack[-1].tag == 'symbol':
                 if (len(stack[-1].children) != 1 or
-                    not isinstance(stack[-1].children[0], six.string_types)):
+                    not isinstance(stack[-1].children[0], str)):
                     estr = "Invalid symbol code."
                     errors.append(ColorizingError(estr, token, end))
                 else:
@@ -1064,7 +1089,7 @@ def _colorize(doc, token, errors, tagName='para'):
             # Special handling for escape elements:
             if stack[-1].tag == 'escape':
                 if (len(stack[-1].children) != 1 or
-                    not isinstance(stack[-1].children[0], six.string_types)):
+                    not isinstance(stack[-1].children[0], str)):
                     estr = "Invalid escape code."
                     errors.append(ColorizingError(estr, token, end))
                 else:
@@ -1094,8 +1119,8 @@ def _colorize(doc, token, errors, tagName='para'):
         start = end+1
 
     # Add any final text.
-    if start < len(str):
-        stack[-1].children.append(str[start:])
+    if start < len(text):
+        stack[-1].children.append(text[start:])
 
     if len(stack) != 1:
         estr = "Unbalanced '{'."
@@ -1107,8 +1132,8 @@ def _colorize_link(doc, link, token, end, errors):
     variables = link.children[:]
 
     # If the last child isn't text, we know it's bad.
-    if len(variables)==0 or not isinstance(variables[-1], six.string_types):
-        estr = "Bad %s target." % link.tag
+    if len(variables)==0 or not isinstance(variables[-1], str):
+        estr = f"Bad {link.tag} target."
         errors.append(ColorizingError(estr, token, end))
         return
 
@@ -1121,7 +1146,7 @@ def _colorize_link(doc, link, token, end, errors):
     elif len(variables) == 1:
         target = variables[0]
     else:
-        estr = "Bad %s target." % link.tag
+        estr = f"Bad {link.tag} target."
         errors.append(ColorizingError(estr, token, end))
         return
 
@@ -1146,7 +1171,7 @@ def _colorize_link(doc, link, token, end, errors):
             return
 
     # Construct the target element.
-    target_elt = Element('target', target)
+    target_elt = Element('target', target, lineno=str(token.startline))
 
     # Add them to the link element.
     link.children = [name_elt, target_elt]
@@ -1199,7 +1224,7 @@ class ColorizingError(ParseError):
         else:
             right = (self.token.contents[self.charnum:self.charnum+RANGE]
                      + '...')
-        return ('%s\n\n%s%s\n%s^' % (self._descr, left, right, ' '*len(left)))
+        return f"{self._descr}\n\n{left}{right}\n{' '*len(left)}^"
 
 #################################################################
 ##                    SUPPORT FOR EPYDOC
@@ -1296,16 +1321,20 @@ class ParsedEpytextDocstring(ParsedDocstring):
         '<=': 8804, '>=': 8805,
         }
 
-    def __init__(self, body, fields):
+    def __init__(self, body: Optional[Element], fields: Sequence['Field']):
         ParsedDocstring.__init__(self, fields)
         self._tree = body
         # Caching:
-        self._stan = None
+        self._stan: Optional[Tag] = None
 
-    def __str__(self):
+    def __str__(self) -> str:
         return str(self._tree)
 
-    def to_stan(self, docstring_linker):
+    @property
+    def has_body(self) -> bool:
+        return self._tree is not None
+
+    def to_stan(self, docstring_linker: DocstringLinker) -> Tag:
         if self._stan is not None:
             return self._stan
         if self._tree is None:
@@ -1314,8 +1343,12 @@ class ParsedEpytextDocstring(ParsedDocstring):
             self._stan = self._to_stan(self._tree, docstring_linker)
         return self._stan
 
-    def _to_stan(self, tree, linker, seclevel=0):
-        if isinstance(tree, six.string_types):
+    def _to_stan(self,
+            tree: Union[Element, str],
+            linker: DocstringLinker,
+            seclevel: int = 0
+            ) -> Any:
+        if isinstance(tree, str):
             return tree
 
         if tree.tag == 'section':
@@ -1336,8 +1369,9 @@ class ParsedEpytextDocstring(ParsedDocstring):
             return tags.a(variables[0], href=variables[1], target='_top')
         elif tree.tag == 'link':
             label = tags.code(variables[0])
+            lineno = int(tree.children[1].attribs['lineno'])
             try:
-                url = linker.resolve_identifier_xref(variables[1])
+                url = linker.resolve_identifier_xref(variables[1], lineno)
             except LookupError:
                 return label
             else:
@@ -1362,7 +1396,7 @@ class ParsedEpytextDocstring(ParsedDocstring):
         elif tree.tag == 'li':
             return tags.li(*variables)
         elif tree.tag == 'heading':
-            return getattr(tags, 'h%d' % seclevel)(*variables)
+            return getattr(tags, f'h{seclevel:d}')(*variables)
         elif tree.tag == 'literalblock':
             variables.append('\n')
             return tags.pre('\n', *variables, class_='literalblock')
@@ -1376,4 +1410,4 @@ class ParsedEpytextDocstring(ParsedDocstring):
             symbol = tree.children[0]
             return CharRef(self.SYMBOL_TO_CODEPOINT[symbol])
         else:
-            raise AssertionError("Unknown epytext DOM element %r" % tree.tag)
+            raise AssertionError(f"Unknown epytext DOM element {tree.tag!r}")
