@@ -43,6 +43,7 @@ the list.
 __docformat__ = 'epytext en'
 
 # Imports
+from typing import Sequence
 import re
 
 from docutils.core import publish_string
@@ -50,17 +51,17 @@ from docutils.writers import Writer
 from docutils.writers.html4css1 import HTMLTranslator, Writer as HTMLWriter
 from docutils.readers.standalone import Reader as StandaloneReader
 from docutils.utils import new_document
-from docutils.nodes import NodeVisitor, SkipNode
+from docutils.nodes import NodeVisitor, SkipNode, Text
 from docutils.frontend import OptionParser
-from docutils.parsers.rst import directives
+from docutils.parsers.rst import Directive, directives
 import docutils.nodes
 import docutils.transforms.frontmatter
 import docutils.utils
 
-from twisted.web.template import tags
+from twisted.web.template import Tag, tags
 from pydoctor.epydoc.doctest import colorize_codeblock, colorize_doctest
 from pydoctor.epydoc.markup import (
-    Field, ParseError, ParsedDocstring, flatten, html2stan
+    DocstringLinker, Field, ParseError, ParsedDocstring, flatten, html2stan
 )
 from pydoctor.epydoc.markup.plaintext import ParsedPlaintextDocstring
 
@@ -121,29 +122,32 @@ class ParsedRstDocstring(ParsedDocstring):
     An encoded version of a ReStructuredText docstring.  The contents
     of the docstring are encoded in the L{_document} instance
     variable.
-
-    @ivar _document: A ReStructuredText document, encoding the
-        docstring.
-    @type _document: C{docutils.nodes.document}
     """
-    def __init__(self, document, fields):
-        """
-        @type document: C{docutils.nodes.document}
-        """
+
+    def __init__(self, document: docutils.nodes.document, fields: Sequence[Field]):
         self._document = document
+        """A ReStructuredText document, encoding the docstring."""
 
         document.reporter = OptimizedReporter(
             document.reporter.source, 'SEVERE', 'SEVERE', '')
 
         ParsedDocstring.__init__(self, fields)
 
-    def to_stan(self, docstring_linker):
+    @property
+    def has_body(self) -> bool:
+        return any(
+            isinstance(child, Text) or child.children
+            for child in self._document.children
+            )
+
+    def to_stan(self, docstring_linker: DocstringLinker) -> Tag:
         # Inherit docs
         visitor = _EpydocHTMLTranslator(self._document, docstring_linker)
         self._document.walkabout(visitor)
         return html2stan(''.join(visitor.body))
 
-    def __repr__(self): return '<ParsedRstDocstring: ...>'
+    def __repr__(self) -> str:
+        return '<ParsedRstDocstring: ...>'
 
 class _EpydocReader(StandaloneReader):
     """
@@ -241,7 +245,7 @@ class _SplitFieldsTranslator(NodeVisitor):
                         return
                     except ValueError as e:
                         estr = 'Unable to split consolidated field '
-                        estr += '"%s" - %s' % (tagname, e)
+                        estr += f'"{tagname}" - {e}'
                         self._errors.append(ParseError(estr, node.line,
                                                        is_fatal=False))
 
@@ -393,8 +397,11 @@ class _EpydocHTMLTranslator(HTMLTranslator):
         if m: text, target = m.groups()
         else: target = text = node.astext()
         label = tags.code(text)
+        # TODO: 'node.line' is None for some reason.
+        #       https://github.com/twisted/pydoctor/issues/237
+        lineno = 0
         try:
-            url = self._linker.resolve_identifier_xref(target)
+            url = self._linker.resolve_identifier_xref(target, lineno)
         except LookupError:
             xref = label
         else:
@@ -435,12 +442,12 @@ class _EpydocHTMLTranslator(HTMLTranslator):
                 # Prefix all CSS classes with "rst-"; and prefix all
                 # names with "rst-" to avoid conflicts.
                 if key.lower() in ('class', 'id', 'name'):
-                    attr_dict[key] = 'rst-%s' % val
+                    attr_dict[key] = f'rst-{val}'
                 elif key.lower() in ('classes', 'ids', 'names'):
-                    attr_dict[key] = ['rst-%s' % cls for cls in val]
+                    attr_dict[key] = [f'rst-{cls}' for cls in val]
                 elif key.lower() == 'href':
                     if attr_dict[key][:1]=='#':
-                        attr_dict[key] = '#rst-%s' % attr_dict[key][1:]
+                        attr_dict[key] = f'#rst-{attr_dict[key][1:]}'
                     else:
                         # If it's an external link, open it in a new
                         # page.
@@ -462,8 +469,7 @@ class _EpydocHTMLTranslator(HTMLTranslator):
             self.body.append(flatten(colorize_doctest(pysrc)))
         raise SkipNode()
 
-def python_code_directive(name, arguments, options, content, lineno,
-                          content_offset, block_text, state, state_machine):
+class PythonCodeDirective(Directive):
     """
     A custom restructuredtext directive which can be used to display
     syntax-highlighted Python code blocks.  This directive takes no
@@ -473,11 +479,12 @@ def python_code_directive(name, arguments, options, content, lineno,
     prefer that the output not contain prompts (e.g., to make
     copy/paste easier).
     """
-    text = '\n'.join(content)
-    node = docutils.nodes.doctest_block(text, text, codeblock=True)
-    return [ node ]
 
-python_code_directive.arguments = (0, 0, 0)
-python_code_directive.content = True
+    has_content = True
 
-directives.register_directive('python', python_code_directive)
+    def run(self):
+        text = '\n'.join(self.content)
+        node = docutils.nodes.doctest_block(text, text, codeblock=True)
+        return [ node ]
+
+directives.register_directive('python', PythonCodeDirective)
