@@ -1,55 +1,35 @@
 """The classes that turn  L{Documentable} instances into objects we can render."""
 
-from __future__ import print_function
+from typing import Any, Iterator, List, Optional, Union
+import ast
 
-from twisted.web.template import tags, Element, renderer, XMLFile
+from twisted.web.template import tags, Element, renderer, Tag, XMLFile
+import astor
 
-from pydoctor import epydoc2stan, model
+from pydoctor import epydoc2stan, model, __version__
+from pydoctor.astbuilder import node2fullname
 from pydoctor.templatewriter.pages.table import ChildTable
 from pydoctor.templatewriter import util
 
-def getBetterThanArgspec(argspec):
-    """Ok, maybe argspec's format isn't the best after all: This takes an
-    argspec and returns (regularArguments, [(kwarg, kwval), (kwarg, kwval)])."""
-    args = argspec[0]
-    defaults = argspec[-1]
-    if not defaults:
-        return (args, [])
-    backargs = args[:]
-    backargs.reverse()
-    defaults = list(defaults)
-    defaults.reverse()
-    kws = list(zip(backargs, defaults))
-    kws.reverse()
-    return (args[:-len(kws)], kws)
 
-def _strtup(tup):
-    # Ugh
-    if not isinstance(tup, (tuple, list)):
-        return str(tup)
-    return '(' + ', '.join(map(_strtup, tup)) + ')'
+def format_decorators(obj: Union[model.Function, model.Attribute]) -> Iterator[Any]:
+    for dec in obj.decorators or ():
+        if isinstance(dec, ast.Call):
+            fn = node2fullname(dec.func, obj)
+            # We don't want to show the deprecated decorator;
+            # it shows up as an infobox.
+            if fn in ("twisted.python.deprecate.deprecated",
+                      "twisted.python.deprecate.deprecatedProperty"):
+                break
 
-def signature(argspec):
-    """Return a nicely-formatted source-like signature, formatted from an
-    argspec.
-    """
-    regargs, kwargs = getBetterThanArgspec(argspec)
-    varargname, varkwname = argspec[1:3]
-    things = []
-    for regarg in regargs:
-        if isinstance(regarg, list):
-            things.append(_strtup(regarg))
-        else:
-            things.append(regarg)
-    if varargname:
-        things.append('*%s' % varargname)
+        text = '@' + astor.to_source(dec).strip()
+        yield text, tags.br()
 
-    things += ['%s=%s' % kwarg for kwarg in kwargs]
-    if varkwname:
-        things.append('**%s' % varkwname)
-    return ', '.join(things)
+def signature(function: model.Function) -> str:
+    """Return a nicely-formatted source-like function signature."""
+    return str(function.signature)
 
-class DocGetter(object):
+class DocGetter:
     def get(self, ob, summary=False):
         if summary:
             return epydoc2stan.format_summary(ob)
@@ -81,23 +61,31 @@ class CommonPage(Element):
 
     def heading(self):
         return tags.h1(class_=self.ob.css_class)(
-            tags.code(self.mediumName(self.ob)), " ",
-            tags.small(self.ob.kind.lower(), " documentation"),
+            tags.code(self.namespace(self.ob))
             )
 
+    def category(self) -> str:
+        return f"{self.ob.kind.lower()} documentation"
+
+    def namespace(self, obj: model.Documentable) -> List[Union[Tag, str]]:
+        parts: List[Union[Tag, str]] = []
+        ob: Optional[model.Documentable] = obj
+        while ob:
+            if ob.documentation_location is model.DocLocation.OWN_PAGE:
+                if parts:
+                    parts.append('.')
+                parts.append(util.taglink(ob, ob.name))
+            ob = ob.parent
+        parts.reverse()
+        return parts
+
+    # Deprecated: pydoctor's templates no longer use this, but it is kept
+    #             for now to not break customized templates like Twisted's.
+    #             NOTE: Remember to remove the CSS as well.
     def part(self):
-        if self.ob.parent:
-            parent = self.ob.parent
-            if isinstance(parent, model.Module) and parent.name == '__init__':
-                parent = parent.parent
-            parts = []
-            while parent.parent:
-                parts.append(util.taglink(parent, parent.name))
-                parts.append('.')
-                parent = parent.parent
-            parts.append(util.taglink(parent, parent.name))
-            parts.reverse()
-            return 'Part of ', tags.code(parts)
+        parent = self.ob.parent
+        if parent:
+            return 'Part of ', tags.code(self.namespace(parent))
         else:
             return []
 
@@ -177,6 +165,7 @@ class CommonPage(Element):
         return tag.fillSlots(
             title=self.title(),
             heading=self.heading(),
+            category=self.category(),
             part=self.part(),
             extras=self.extras(),
             docstring=self.docstring(),
@@ -184,13 +173,25 @@ class CommonPage(Element):
             packageInitTable=self.packageInitTable(),
             childlist=self.childlist(),
             project=self.project(),
+            version=__version__.public(),
             buildtime=self.ob.system.buildtime.strftime("%Y-%m-%d %H:%M:%S"))
 
 
-class PackagePage(CommonPage):
+class ModulePage(CommonPage):
+    def extras(self):
+        r = super().extras()
+
+        sourceHref = util.srclink(self.ob)
+        if sourceHref:
+            r.append(tags.a("(source)", href=sourceHref))
+
+        return r
+
+
+class PackagePage(ModulePage):
     def children(self):
-        return sorted([o for o in self.ob.contents.values()
-                       if o.name != '__init__' and o.isVisible],
+        return sorted((o for o in self.ob.contents.values()
+                       if o.name != '__init__' and o.isVisible),
                       key=lambda o2:(-o2.privacyClass.value, o2.fullName()))
 
     def packageInitTable(self):
@@ -210,8 +211,6 @@ class PackagePage(CommonPage):
                 if o.documentation_location is model.DocLocation.PARENT_PAGE
                 and o.isVisible]
 
-class ModulePage(CommonPage):
-    pass
 
 def overriding_subclasses(c, name, firstcall=True):
     if not firstcall and name in c.contents:
@@ -219,8 +218,7 @@ def overriding_subclasses(c, name, firstcall=True):
     else:
         for sc in c.subclasses:
             if sc.isVisible:
-                for sc2 in overriding_subclasses(sc, name, False):
-                    yield sc2
+                yield from overriding_subclasses(sc, name, False)
 
 def nested_bases(b):
     r = [(b,)]
@@ -232,9 +230,11 @@ def nested_bases(b):
     return r
 
 def unmasked_attrs(baselist):
-    maybe_masking = set()
-    for b in baselist[1:]:
-        maybe_masking.update(set([o.name for o in b.contents.values()]))
+    maybe_masking = {
+        o.name
+        for b in baselist[1:]
+        for o in b.contents.values()
+        }
     return [o for o in baselist[0].contents.values()
             if o.isVisible and o.name not in maybe_masking]
 
@@ -276,7 +276,18 @@ class ClassPage(CommonPage):
         self.overridenInCount = 0
 
     def extras(self):
-        r = super(ClassPage, self).extras()
+        r = super().extras()
+
+        sourceHref = util.srclink(self.ob)
+        if sourceHref:
+            source = (" ", tags.a("(source)", href=sourceHref))
+        else:
+            source = tags.transparent
+        r.append(tags.p(tags.code(
+            tags.span("class", class_='py-keyword'), " ",
+            self.mediumName(self.ob), ":", source
+            )))
+
         scs = sorted(self.ob.subclasses, key=lambda o:o.fullName().lower())
         if not scs:
             return r
@@ -287,7 +298,7 @@ class ClassPage(CommonPage):
         return r
 
     def mediumName(self, ob):
-        r = [super(ClassPage, self).mediumName(ob)]
+        r = [super().mediumName(ob)]
         zipped = list(zip(self.ob.rawbases, self.ob.bases, self.ob.baseobjects))
         if zipped:
             r.append('(')
@@ -353,9 +364,11 @@ class ClassPage(CommonPage):
 
 class ZopeInterfaceClassPage(ClassPage):
     def extras(self):
-        r = [super(ZopeInterfaceClassPage, self).extras()]
+        r = [super().extras()]
         if self.ob.isinterface:
-            namelist = sorted([o.fullName() for o in self.ob.implementedby_directly], key=lambda x:x.lower())
+            namelist = sorted(
+                    (o.fullName() for o in self.ob.implementedby_directly),
+                    key=lambda x:x.lower())
             label = 'Known implementations: '
         else:
             namelist = sorted(self.ob.implements_directly, key=lambda x:x.lower())
@@ -381,11 +394,9 @@ class ZopeInterfaceClassPage(ClassPage):
         r = []
         if imeth:
             r.append(tags.div(class_="interfaceinfo")('from ', util.taglink(imeth, imeth.parent.fullName())))
-        r.extend(super(ZopeInterfaceClassPage, self).functionExtras(data))
+        r.extend(super().functionExtras(data))
         return r
 
 class FunctionPage(CommonPage):
     def mediumName(self, ob):
-        return [
-            super(FunctionPage, self).mediumName(ob), '(',
-            signature(self.ob.argspec), ')']
+        return [super().mediumName(ob), signature(self.ob)]
