@@ -1,7 +1,7 @@
-from __future__ import print_function
+from twisted.web.template import Element, XMLFile, renderer, tags
 
 from pydoctor.templatewriter import util
-from twisted.web.template import Element, XMLFile, renderer
+from pydoctor.templatewriter.pages import format_decorators
 
 
 class AttributeChild(Element):
@@ -28,6 +28,10 @@ class AttributeChild(Element):
     def shortFunctionAnchor(self, request, tag):
         return self.ob.name
 
+    @renderer
+    def decorator(self, request, tag):
+        return list(format_decorators(self.ob))
+
     @renderer
     def attribute(self, request, tag):
         return self.ob.name
@@ -46,3 +50,10 @@ class AttributeChild(Element):
     @renderer
     def functionBody(self, request, tag):
         return self.docgetter.get(self.ob)
+
+    @renderer
+    def functionDeprecated(self, request, tag):
+        if hasattr(self.ob, "_deprecated_info"):
+            return (tags.div(self.ob._deprecated_info, role="alert", class_="deprecationNotice alert alert-warning"),)
+        else:
+            return ()
