@@ -1,31 +1,29 @@
-from __future__ import print_function
-
-import os
-import shutil
-import tempfile
 from io import BytesIO
+from pathlib import Path
+from typing import Callable
 
 import pytest
 from pydoctor import model, templatewriter
 from pydoctor.templatewriter import pages, writer
+from pydoctor.templatewriter.summary import isClassNodePrivate, isPrivate
 from pydoctor.test.test_astbuilder import fromText
 from pydoctor.test.test_packages import processPackage
 
 
-def flatten(t):
+def flatten(t: pages.ChildTable) -> str:
     io = BytesIO()
     writer.flattenToFile(io, t)
     return io.getvalue().decode()
 
 
-def getHTMLOf(ob):
+def getHTMLOf(ob: model.Documentable) -> str:
     wr = templatewriter.TemplateWriter('')
-    wr.system = ob.system
     f = BytesIO()
-    wr.writeDocsForOne(ob, f)
+    wr._writeDocsForOne(ob, f)
     return f.getvalue().decode()
 
-def test_simple():
+
+def test_simple() -> None:
     src = '''
     def f():
         """This is a docstring."""
@@ -34,19 +32,19 @@ def test_simple():
     v = getHTMLOf(mod.contents['f'])
     assert 'This is a docstring' in v
 
-def test_empty_table():
+def test_empty_table() -> None:
     mod = fromText('')
     t = pages.ChildTable(pages.DocGetter(), mod, [])
     flattened = flatten(t)
     assert 'The renderer named' not in flattened
 
-def test_nonempty_table():
+def test_nonempty_table() -> None:
     mod = fromText('def f(): pass')
     t = pages.ChildTable(pages.DocGetter(), mod, mod.contents.values())
     flattened = flatten(t)
     assert 'The renderer named' not in flattened
 
-def test_rest_support():
+def test_rest_support() -> None:
     system = model.System()
     system.options.docformat = 'restructuredtext'
     system.options.verbosity = 4
@@ -58,32 +56,29 @@ def test_rest_support():
     html = getHTMLOf(mod.contents['f'])
     assert "<pre>" not in html
 
-def test_document_code_in_init_module():
+def test_document_code_in_init_module() -> None:
     system = processPackage("codeininit")
     html = getHTMLOf(system.allobjects['codeininit'])
     assert 'functionInInit' in html
 
-def test_basic_package():
+def test_basic_package(tmp_path: Path) -> None:
     system = processPackage("basic")
-    targetdir = tempfile.mkdtemp()
-    try:
-        w = writer.TemplateWriter(targetdir)
-        w.system = system
-        system.options.htmlusesplitlinks = True
-        system.options.htmlusesorttable = True
-        w.prepOutputDirectory()
-        root, = system.rootobjects
-        w.writeDocsFor(root, False)
-        w.writeModuleIndex(system)
-        for ob in system.allobjects.values():
-            if ob.documentation_location is model.DocLocation.OWN_PAGE:
-                assert os.path.isfile(os.path.join(targetdir, ob.fullName() + '.html'))
-        with open(os.path.join(targetdir, 'basic.html')) as f:
-            assert 'Package docstring' in f.read()
-    finally:
-        shutil.rmtree(targetdir)
+    w = writer.TemplateWriter(str(tmp_path))
+    system.options.htmlusesplitlinks = True
+    system.options.htmlusesorttable = True
+    w.prepOutputDirectory()
+    root, = system.rootobjects
+    w._writeDocsFor(root, False)
+    w.writeModuleIndex(system)
+    for ob in system.allobjects.values():
+        url = ob.url
+        if '#' in url:
+            url = url[:url.find('#')]
+        assert (tmp_path / url).is_file()
+    with open(tmp_path / 'basic.html') as f:
+        assert 'Package docstring' in f.read()
 
-def test_hasdocstring():
+def test_hasdocstring() -> None:
     system = processPackage("basic")
     from pydoctor.templatewriter.summary import hasdocstring
     assert not hasdocstring(system.allobjects['basic._private_mod'])
@@ -91,7 +86,7 @@ def test_hasdocstring():
     sub_f = system.allobjects['basic.mod.D.f']
     assert hasdocstring(sub_f) and not sub_f.docstring
 
-def test_missing_variable():
+def test_missing_variable() -> None:
     mod = fromText('''
     """Module docstring.
 
@@ -106,7 +101,7 @@ def test_missing_variable():
     'className',
     ['NewClassThatMultiplyInherits', 'OldClassThatMultiplyInherits'],
 )
-def test_multipleInheritanceNewClass(className):
+def test_multipleInheritanceNewClass(className: str) -> None:
     """
     A class that has multiple bases has all methods in its MRO
     rendered.
@@ -123,3 +118,42 @@ def test_multipleInheritanceNewClass(className):
 
     assert "methodA" in html
     assert "methodB" in html
+
+
+@pytest.mark.parametrize('func', [isPrivate, isClassNodePrivate])
+def test_isPrivate(func: Callable[[model.Class], bool]) -> None:
+    """A documentable object is private if it is private itself or
+    lives in a private context.
+    """
+    mod = fromText('''
+    class Public:
+        class Inner:
+            pass
+    class _Private:
+        class Inner:
+            pass
+    ''')
+    public = mod.contents['Public']
+    assert not func(public)
+    assert not func(public.contents['Inner'])
+    private = mod.contents['_Private']
+    assert func(private)
+    assert func(private.contents['Inner'])
+
+
+def test_isClassNodePrivate() -> None:
+    """A node for a private class with public subclasses is considered public."""
+    mod = fromText('''
+    class _BaseForPublic:
+        pass
+    class _BaseForPrivate:
+        pass
+    class Public(_BaseForPublic):
+        pass
+    class _Private(_BaseForPrivate):
+        pass
+    ''')
+    assert not isClassNodePrivate(mod.contents['Public'])
+    assert isClassNodePrivate(mod.contents['_Private'])
+    assert not isClassNodePrivate(mod.contents['_BaseForPublic'])
+    assert isClassNodePrivate(mod.contents['_BaseForPrivate'])
